/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.generator.tal;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import mir.generator.tal.interfaces.TALExpressionParser;
import mir.generator.tal.template.CompositeTemplateNode;
import mir.generator.tal.template.PlainTextTemplateNode;
import mir.generator.tal.template.Template;
import mir.generator.tal.template.TemplateNode;
import mir.generator.tal.template.TemplateNodeLibrary;
import mir.generator.tal.template.TemplateNodeLibraryRegistry;
import mir.util.HTMLRoutines;
import mir.util.xml.SectionHandler;
import mir.util.xml.XMLName;
import mir.util.xml.XMLParserEngine;
import mir.util.xml.XMLParserExc;
import mir.util.xml.XMLParserFailure;
import mir.util.xml.XMLReaderTool;

public class TALTemplateParser {
  private TemplateNodeLibraryRegistry registry;

  public TALTemplateParser(TemplateNodeLibraryRegistry aRegistry) {
    registry = aRegistry;
  }

  public Template parse(String aData, TALExpressionParser aParser) throws TALExc, TALFailure {
    return parse(new StringReader(aData), aParser);
  }

  public Template parse(File aFile, TALExpressionParser aParser) throws TALExc, TALFailure {
    try {
      return parse(new BufferedInputStream(new FileInputStream(aFile), 1024*128), aParser);
    }
    catch (FileNotFoundException e) {
      throw new TALFailure(e);
    }
  }
  public Template parse(InputStream anInputStream, TALExpressionParser aParser) throws TALExc, TALFailure {
    return parse(new InputStreamReader(anInputStream), aParser);
  }

  public Template parse(Reader aReader, TALExpressionParser aParser) throws TALExc, TALFailure {
    Map templateContext = new HashMap();
    TALHandler handler = new TALHandler(aParser, templateContext);

    try {
      XMLParserEngine.getInstance().parse("html", aReader, handler);
    }
    catch (XMLParserExc e) {
      throw new TALFailure(e);
    }

    return new Template(aParser, handler.getNode(), templateContext);
  }

  protected class TALHandler implements SectionHandler {
    private CompositeTemplateNode compositeNode;
    private StringBuffer data;
    private StringBuffer plainData;
    private TALExpressionParser parser;
    private String currentTag;
    private TemplateNodeLibrary library;
    private XMLName tag;
    private Map attributes;
    private Map templateContext;

    public TALHandler(TALExpressionParser aParser, Map aTemplateContext) {
      parser = aParser;
      data = new StringBuffer();
      plainData = new StringBuffer();
      compositeNode = new CompositeTemplateNode();
      templateContext = aTemplateContext;
    }

    private void flushData() {
      if (data.length()!=0) {
        compositeNode.appendSubNode(new PlainTextTemplateNode(data.toString(), plainData.toString()));
        data.delete(0, data.length());
        plainData.delete(0, plainData.length());
      }
    }

    public void extra(String anExtraData) throws XMLParserExc, XMLParserFailure {
      appendCode(anExtraData);
    }

    public TemplateNodeLibrary findLibrary(XMLName aName) {
      TemplateNodeLibrary result = null;

      if (aName.getNamespaceURI()!=null) {
        result = registry.findLibraryForUrl(aName.getNamespaceURI());
      }

      if ((result == null) && (aName.getPrefix()!=null) && (aName.getPrefix().length()>0)) {
        result = registry.findLibraryForPrefix(aName.getPrefix());
      }

      return result;
    }

    public SectionHandler startElement(mir.util.xml.XMLName aTag, Map anAttributes) throws XMLParserExc {
      library = findLibrary(aTag);

      Iterator i = anAttributes.keySet().iterator();
      while (library==null && i.hasNext()) {
        library=findLibrary((XMLName) i.next());
      }

      currentTag = XMLReaderTool.normalizeXMLName(aTag);

      if (library == null) {
        appendCode("<"+currentTag);
        i = anAttributes.entrySet().iterator();

        while (i.hasNext()) {
          Map.Entry entry = (Map.Entry) i.next();

          appendCode(" "+ XMLReaderTool.normalizeXMLName((XMLName) entry.getKey()));
          appendCode("=\"");
          appendText((String) entry.getValue());
          appendCode("\"");
        }
      }
      else {
        tag = aTag;
        attributes = anAttributes;
      }

      return new TALHandler(parser, templateContext);
    }

    public void endElement(mir.util.xml.SectionHandler aHandler) throws XMLParserExc {
      if (library == null) {
        TemplateNode subNode = ((TALHandler) aHandler).getNode();
        if (subNode instanceof CompositeTemplateNode &&
            ((CompositeTemplateNode) subNode).isEmpty()) {
          appendCode(" />");
        }
        else {
          appendCode(">");
          appendSubNode(subNode);
          appendCode("</"+currentTag+">");
        }
      }
      else {
        appendSubNode(
            library.constructTemplateNode(parser, tag, attributes, ((TALHandler) aHandler).getNode(), templateContext));
        tag = null;
        attributes = null;
      }
    }

    protected void appendSubNode(TemplateNode aNode) {
      flushData();

      compositeNode.appendSubNode(aNode);
    }

    protected void appendCode(String aCode) {
      data.append(aCode);
    }

    protected void appendText(String aText) {
      data.append(HTMLRoutines.encodeHTML(aText));
      plainData.append(aText);
    }

    public void finishSection() throws XMLParserExc {
      flushData();
    }

    public TemplateNode getNode() {
      return compositeNode;
    }

    public void characters(String aCharacters) throws XMLParserExc {
      appendText(aCharacters);
    }

    public void startSection() throws XMLParserExc, XMLParserFailure {
    }
  }
}
