/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.util;

import mir.log.LoggerWrapper;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;

/**
 * Execute system commands. Warning: the current implementation is
 * unix specific.
 */
public class ShellRoutines {
  protected ShellRoutines() {
  }

  /**
   * Executes a full command (including arguments) in a subshell
   * and returns the output of the command in a string. Output is
   * redirected into a temporary fil which is then read into the string
   */
  public static String execIntoString(String command) throws IOException {
    return new String(execIntoByteArray(command));
  }

  /**
   * Executes a full command (including arguments) in a subshell
   * and returns the output of the command in an array of
   * bytes. Output is redirected into a temporary file which is then
   * read into an array of bytes
   */
  public static byte[] execIntoByteArray(String command) throws IOException {
    File commandOutput = File.createTempFile("mircmd", "");
    try {
      int exitStatus;
      try {
        // WARNING: unix specific

        exitStatus = Runtime.getRuntime().exec(new String[]{
          "/bin/sh", "-c",
          command + " " +
            ">" + commandOutput.getAbsolutePath()
        }).waitFor();
      }
      catch (InterruptedException e) {
        throw new IOException(e.toString());
      }
      if (exitStatus != 0) {
        throw new IOException("command exit satus:" + exitStatus);
      }
      return FileRoutines.readFileIntoByteArray
          (commandOutput.getAbsoluteFile());
    }
    finally {
      commandOutput.delete();
    }
  }

  /**
   * Executes a full command (including arguments) in a subshell.
   * Standard input and output go to /dev/null
   */
  public static void simpleExec(String command) throws IOException {
    int exitStatus;
    try {
      // WARNING: unix specific
//      exitStatus =
      Process process = Runtime.getRuntime().exec(
           new String[]{
               "/bin/sh", "-c", command
      });
      Reader errorStreamReader = new InputStreamReader(process.getErrorStream());
      ReaderLogger logger =
          new ReaderLogger(errorStreamReader, new LoggerWrapper("Utility"));

      new Thread(logger).start();

      Reader outputStreamReader = new InputStreamReader(process.getInputStream());
      logger = new ReaderLogger(outputStreamReader, new LoggerWrapper("Utility"));
      new Thread(logger).start();

      exitStatus = process.waitFor();
    }
    catch (InterruptedException e) {
      throw new IOException("Interrupted");
    }

    if (exitStatus != 0) {
      throw new IOException("command exit satus:" + exitStatus);
    }
  }

  private static class ReaderLogger implements Runnable {
    private Reader reader;
    private PrintWriter writer;

    ReaderLogger(Reader aReader, LoggerWrapper aLogger) {
      reader = aReader;
      writer = aLogger.asPrintWriter(LoggerWrapper.DEBUG_MESSAGE);
    }

    public void run() {
      try {
        int size;
        char[] buffer = new char[1024];
        while ((size = reader.read(buffer)) >= 0) {
          writer.write(buffer,0,size);
        }
      }
      catch (IOException e) {
      }

      writer.close();
      try {
        reader.close();
      }
      catch (Exception e) {
      }
    }
  }

}
